/// Class representing current weather forecast value.
class CurrentWeather {
  Coordinate coord;
  List<Weather> weather;
  String base;
  MainWeatherData main;
  int visibility;
  Wind wind;
  Clouds clouds;
  int dt;
  Sys sys;
  int timezone;
  int id;
  String name;
  int cod;

  CurrentWeather({
    this.coord,
    this.weather,
    this.base,
    this.main,
    this.visibility,
    this.wind,
    this.clouds,
    this.dt,
    this.sys,
    this.timezone,
    this.id,
    this.name,
    this.cod,
  });

  CurrentWeather.fromJSON(Map<String, dynamic> data)
      : coord = new Coordinate.fromJSON(data['coord']),
        weather = List<Weather>.from(
            data['weather'].map((w) => new Weather.fromJSON(w))),
        base = data['base'],
        main = new MainWeatherData.fromJSON(data['main']),
        visibility = data['visibility'],
        wind = new Wind.fromJSON(data['wind']),
        clouds = new Clouds.fromJSON(data['clouds']),
        dt = data['dt'],
        sys = new Sys.fromJSON(data['sys']),
        timezone = data['timezone'],
        id = data['id'],
        name = data['name'],
        cod = data['cod'];
}

/// Class representing clould data found in current weather forecast.
class Clouds {
  int all;

  Clouds({
    this.all,
  });
  Clouds.fromJSON(Map<String, dynamic> data) : all = data['all'];
}

/// Simple coordinate data.
class Coordinate {
  double longitude;
  double latitude;

  double get lat => this.latitude;
  double get lon => this.longitude;

  Coordinate({
    this.longitude,
    this.latitude,
  });

  Coordinate.fromJSON(Map<String, dynamic> data)
      : longitude = data['lon'],
        latitude = data['lat'];
}

/// Main weather data in numbers instead of pretty human readable form.
class MainWeatherData {
  double temperature;
  double pressure;
  int humidity;
  double temperatureMin;
  double temperatureMax;

  double get temp => this.temperature;
  double get tempMax => this.temperatureMax;
  double get tempMin => this.temperatureMin;

  MainWeatherData({
    this.temperature,
    this.pressure,
    this.humidity,
    this.temperatureMin,
    this.temperatureMax,
  });

  MainWeatherData.fromJSON(Map<String, dynamic> data)
      : temperature = data['temp'].toDouble(),
        pressure = data['pressure'].toDouble(),
        humidity = data['humidity'],
        temperatureMax = data['temp_max'].toDouble(),
        temperatureMin = data['temp_min'].toDouble();
}

/// 'Sys' data, don't know the full form.
class Sys {
  int type;
  int id;
  double message;
  String country;
  int sunrise;
  int sunset;

  Sys({
    this.type,
    this.id,
    this.message,
    this.country,
    this.sunrise,
    this.sunset,
  });

  Sys.fromJSON(Map<String, dynamic> data)
      : type = data['type'],
        id = data['id'],
        message = data['message'].toDouble(),
        country = data['country'],
        sunrise = data['sunrise'],
        sunset = data['sunset'];
}

/// Human readable weather data.
class Weather {
  int id;
  String main;
  String description;
  String icon;

  Weather({
    this.id,
    this.main,
    this.description,
    this.icon,
  });

  Weather.fromJSON(Map<String, dynamic> data)
      : id = data['id'],
        main = data['main'],
        description = data['description'],
        icon = data['icon'];
}

/// Wind data details.
class Wind {
  double speed;
  double degree;

  double get deg => this.degree;

  Wind({
    this.speed,
    this.degree,
  });

  Wind.fromJSON(Map<String, dynamic> data)
      : speed = data['speed'].toDouble(),
        degree = data['deg'].toDouble();
}
